import numpy as np
cimport numpy as np
from . import substitution_matrices as submat
from libc.string cimport strlen


# Access to the Python/C API
cdef extern from 'Python.h':
    ctypedef void PyObject
    PyObject *PyString_FromStringAndSize(char *, size_t)
    int _PyString_Resize(PyObject **, size_t)
    char * PyString_AS_STRING(PyObject *)


# Declaring numpy data types speeds things up massively
ctypedef np.int_t DTYPE_INT
ctypedef np.uint_t DTYPE_UINT
ctypedef np.float32_t DTYPE_FLOAT


cdef inline DTYPE_FLOAT max3(DTYPE_FLOAT a, DTYPE_FLOAT b, DTYPE_FLOAT c):
    '''Find largest of 3 floats. Much faster than using built-in max.

    :param a: First number.
    :type a: DTYPE_FLOAT
    :param b: Second number.
    :type b: DTYPE_FLOAT
    :param c: Third number.
    :type c: DTYPE_FLOAT

    '''
    if c > b:
        return c if c > a else a
    return b if b > a else a


cdef inline DTYPE_FLOAT max2(DTYPE_FLOAT a, DTYPE_FLOAT b):
    '''Find largest of 2 floats. Much faster than using built-in max and max3.

    :param a: First number.
    :type a: DTYPE_FLOAT
    :param b: Second number.
    :type b: DTYPE_FLOAT

    '''
    return b if b > a else a


def as_ord_matrix(matrix):
    '''Given the SubstitutionMatrix input, generate an equivalent matrix that
    is indexed by the ASCII number of each residue (e.g. A -> 65).'''
    ords = [ord(c) for c in matrix.alphabet]
    ord_matrix = np.zeros((max(ords) + 1, max(ords) + 1), dtype=np.integer)
    for i, row_ord in enumerate(ords):
        for j, col_ord in enumerate(ords):
            ord_matrix[row_ord, col_ord] = matrix[i, j]

    return ord_matrix


def max_index(array):
    '''Locate the index of the largest value in the array. If there are
    multiple, finds the earliest one in the row-flattened array.

    :param array: Any array.
    :type array: numpy.array

    '''
    return np.unravel_index(array.argmax(), array.shape)


def aligner(_seqj, _seqi, DTYPE_FLOAT gap_open=-7, DTYPE_FLOAT gap_extend=-7,
            DTYPE_FLOAT gap_double=-7, method='global',
            matrix=submat.DNA_SIMPLE):
    '''Calculates the alignment of two sequences. The global method uses
    a global Needleman-Wunsh algorithm, local does a a local
    Smith-Waterman alignment, global_cfe does a global alignment with
    cost-free ends and glocal does an alignment which is global only with
    respect to the shorter sequence, also known as a semi-global
    alignment. Returns the aligned (sub)sequences as character arrays.

    Gotoh, O. (1982). J. Mol. Biol. 162, 705-708.
    Needleman, S. & Wunsch, C. (1970). J. Mol. Biol. 48(3), 443-53.
    Smith, T.F. & Waterman M.S. (1981). J. Mol. Biol. 147, 195-197.

    :param seqj: First sequence.
    :type seqj: str
    :param seqi: Second sequence.
    :type seqi: str
    :param method: Type of alignment: 'global', 'global_cfe', 'local', or
                   'glocal'.
    :type method: str
    :param gap_open: The cost of opening a gap (negative number).
    :type gap_open: float
    :param gap_extend: The cost of extending an open gap (negative number).
    :type gap_extend: float
    :param gap_double: The gap-opening cost if a gap is already open in the
                       other sequence (negative number).
    :type gap_double: float
    :param matrix: A score matrix dictionary name. Examples can be found in
                   the substitution_matrices module.
    :type matrix: SubstitutionMatrix

    '''
    cdef int NONE = 0,  LEFT = 1, UP = 2,  DIAG = 3
    cdef bint flip = 0
    cdef char* seqj = _seqj
    cdef char* seqi = _seqi
    cdef size_t align_counter = 0

    cdef int imethod

    if method == 'global':
        imethod = 0
    elif method == 'local':
        imethod = 1
    elif method == 'glocal':
        imethod = 2
    elif method == 'global_cfe':
        imethod = 3

    cdef size_t max_j = strlen(seqj)
    cdef size_t max_i = strlen(seqi)
    if max_i == max_j == 0:
        return '', ''

    if max_j > max_i:
        flip = 1
        seqi, seqj = seqj, seqi
        max_i, max_j = max_j, max_i

    cdef char *align_j, *align_i
    cdef int i, j
    cdef char ci, cj
    cdef PyObject *ai, *aj

    assert gap_extend <= 0, 'gap_extend penalty must be <= 0'
    assert gap_open <= 0, 'gap_open must be <= 0'

    cdef np.ndarray[DTYPE_FLOAT, ndim=2] agap_i = np.empty((max_i + 1, max_j + 1), dtype=np.float32)
    cdef np.ndarray[DTYPE_FLOAT, ndim=2] agap_j = np.empty((max_i + 1, max_j + 1), dtype=np.float32)
    agap_i.fill(-np.inf)
    agap_j.fill(-np.inf)

    cdef np.ndarray[DTYPE_FLOAT, ndim=2] score = np.zeros((max_i + 1, max_j + 1), dtype=np.float32)

    cdef np.ndarray[DTYPE_UINT, ndim=2] pointer = np.zeros((max_i + 1, max_j + 1), dtype=np.uint)
    cdef np.ndarray[DTYPE_INT, ndim=2] amatrix = matrix
    amatrix = as_ord_matrix(matrix)

    # START HERE:
    if imethod == 0:
        pointer[0, 1:] = LEFT
        pointer[1:, 0] = UP
        score[0, 1:] = gap_open + gap_extend * np.arange(0, max_j, dtype=np.float32)
        score[1:, 0] = gap_open + gap_extend * np.arange(0, max_i, dtype=np.float32)
    elif imethod == 3:
        pointer[0, 1:] = LEFT
        pointer[1:, 0] = UP
    elif imethod == 2:
        pointer[0, 1:] = LEFT
        score[0, 1:] = gap_open + gap_extend * np.arange(0, max_j, dtype=np.float32)

    for i in range(1, max_i + 1):
        ci = seqi[i - 1]
        for j in range(1, max_j + 1):
            cj = seqj[j - 1]
            # agap_i
            agap_i[i,j] = max3(
                         score[i, j - 1] + gap_open,
                         agap_i[i, j - 1] + gap_extend,
                         agap_j[i, j - 1] + gap_double)
            # agap_j
            agap_j[i,j] = max3(
                         score[i - 1, j] + gap_open,
                         agap_j[i - 1, j] + gap_extend,
                         agap_i[i - 1, j] + gap_double)
            # score
            diag_score = score[i - 1, j - 1] + amatrix[ci, cj]
            left_score = agap_i[i, j]
            up_score   = agap_j[i, j]
            max_score = max3(diag_score, up_score, left_score)

            score[i, j] = max_score

            # global
            if max_score == up_score:
                pointer[i,j] = UP
            elif max_score == left_score:
                pointer[i,j] = LEFT
            else:
                pointer[i,j] = DIAG


    if imethod == 0:
        # max anywhere
        i, j = max_index(score)
    elif imethod == 2:
        # max in last col
        i, j = (score[:,-1].argmax(), max_j)
    elif imethod == 3:
        # from i,j to max(max(last row), max(last col)) for free
        row_max, col_idx = score[-1].max(), score[-1].argmax()
        col_max, row_idx = score[:, -1].max(), score[:, -1].argmax()
        if row_max > col_max:
            pointer[-1,col_idx+1:] = LEFT
        else:
            pointer[row_idx+1:,-1] = UP

    seqlen = max_i + max_j
    ai = PyString_FromStringAndSize(NULL, seqlen)
    aj = PyString_FromStringAndSize(NULL, seqlen)

    # use this and PyObject instead of assigning directly...
    align_j = PyString_AS_STRING(aj)
    align_i = PyString_AS_STRING(ai)

    p = pointer[i, j]
    while p != NONE:
        if p == DIAG:
            i -= 1
            j -= 1
            align_j[align_counter] = seqj[j]
            align_i[align_counter] = seqi[i]
        elif p == LEFT:
            j -= 1
            align_j[align_counter] = seqj[j]
            align_i[align_counter] = c'-'
        elif p == UP:
            i -= 1
            align_j[align_counter] = c'-'
            align_i[align_counter] = seqi[i]
        else:
            raise Exception('wtf!:pointer: %i', p)
        align_counter += 1
        p = pointer[i, j]

    _PyString_Resize(&aj, align_counter)
    _PyString_Resize(&ai, align_counter)

    if flip:
        return (<object>ai)[::-1], (<object>aj)[::-1]
    else:
        return (<object>aj)[::-1], (<object>ai)[::-1]


def score_alignment(a, b, int gap_open, int gap_extend, matrix):
    '''Calculate the alignment score from two aligned sequences.

    :param a: The first aligned sequence.
    :type a: str
    :param b: The second aligned sequence.
    :type b: str
    :param gap_open: The cost of opening a gap (negative number).
    :type gap_open: int
    :param gap_extend: The cost of extending an open gap (negative number).
    :type gap_extend: int.
    :param matrix: A score matrix dictionary name. Examples can be found in
                   the substitution_matrices module.
    :type matrix: SubstitutionMatrix

    '''
    cdef char *al = a
    cdef char *bl = b
    cdef size_t l = strlen(al), i
    cdef int score = 0, this_score
    assert strlen(bl) == l, 'Alignment lengths must be the same'
    cdef np.ndarray[DTYPE_INT, ndim=2] mat
    mat = as_ord_matrix(matrix)

    cdef bint gap_started = 0

    for i in range(l):
        if al[i] == c'-' or bl[i] == c'-':
            score += gap_extend if gap_started else gap_open
            gap_started = 1
        else:
            score += mat[al[i], bl[i]]
            gap_started = 0
    return score



import os
from nose.tools import assert_equal, assert_raises, assert_true, assert_false
from coral import reaction, seqio


def test_construction():
    plasmid_path = os.path.join(os.path.dirname(__file__), 'gibson_test.fasta')
    f1_path = os.path.join(os.path.dirname(__file__), 'fragment_1.fasta')
    f2_path = os.path.join(os.path.dirname(__file__), 'fragment_2.fasta')
    f3_path = os.path.join(os.path.dirname(__file__), 'fragment_3.fasta')
    f3_linear_path = os.path.join(os.path.dirname(__file__),
                                  'fragment_3_linear.fasta')
    plasmid = seqio.read_dna(plasmid_path).circularize()
    f1 = seqio.read_dna(f1_path)
    f2 = seqio.read_dna(f2_path)
    f3 = seqio.read_dna(f3_path)
    f3_linear = seqio.read_dna(f3_linear_path)

    gibsoned_circular = reaction.gibson([f1, f2, f3])
    gibsoned_linear = reaction.gibson([f1, f2, f3_linear], linear=True)
#
    expected_length = len(plasmid)
    gibsoned_circular_length = len(gibsoned_circular)
    gibsoned_linear_length = len(gibsoned_linear)
    assert_equal(gibsoned_circular_length, expected_length)
    assert_equal(gibsoned_linear_length, expected_length)
    assert_true(gibsoned_circular.circular)
    assert_false(gibsoned_linear.circular)
    assert(plasmid.is_rotation(gibsoned_circular))
    try:
        assert_equal(str(plasmid), str(gibsoned_linear))
    except AssertionError:
        assert_equal(str(plasmid), str(gibsoned_linear.flip()))

    # Should fail with circular input
    assert_raises(ValueError, reaction.gibson, [f1.circularize()])
    # Should fail if compatible end can't be found
    assert_raises(Exception, reaction.gibson, [f1, f3[50:-50]], linear=True)
    normal = [f1, f2, f3]
    rotated = [f1, f2, f3.reverse_complement()]
    # Gibson should work regardless of fragment orientation
    assert_true(reaction.gibson(normal).is_rotation(reaction.gibson(rotated)))
    # A redundant fragment shouldn't affect the outcome
    assert_equal(reaction.gibson([f1, f2, f3]),
                 reaction.gibson([f1, f2, f2, f3]))
    # A fragment that can't circularize should raise a ValueError
    assert_raises(ValueError, reaction.gibson, [f1, f2, f3[:-80]])
    # But should still work fine as a linear fragment


def test_annotations():
    pass



'''Test dimers submodule of analysis module.'''
from coral import analysis, DNA, Primer
from nose.tools import assert_equal


def test_dimers():
    '''Test dimers function.'''
    anneal_f = DNA('gatcgatcgatacgatcgatatgcgat', ds=False)
    tm_f = 71.86183729637946
    primer_f = Primer(anneal_f, tm_f)

    anneal_r = DNA('atatcgatcatatcgcatatcgatcgtatcgat', ds=False)
    tm_r = 72.14300162714233
    primer_r = Primer(anneal_r, tm_r)

    dimer_output = analysis.dimers(primer_f, primer_r)

    assert_equal(dimer_output, 0.8529446)



'''
Tests for central dogma submodule of reaction module.

'''

from nose.tools import assert_equal, assert_raises
from coral import reaction, DNA, Peptide, RNA


def test_transcription():
    test_dna = DNA('ATGATGGGCAGTGTCGAATTAAATCTGCGTGAGACAGAATTGTGTT' +
                   'TGGGACTACCAGGCGGTGATACAGTTGCACCAGTAACAGGAAACAA' +
                   'AAGAGGATTCTCTGAAACAGTAGATTTGAAACTTAATTTGAACAAT' +
                   'GAGCCAGCCAACAAGGAAGGTTCCACCACTCATGACGTCGTCACAT' +
                   'TTGATAGTAAAGAAAAGAGTGCGTGTCCAAAAGATCCAGCTAAGCC' +
                   'ACCTGCCAAGGCTCAAGTCGTCGGATGGCCACCTGTGAGATCTTAT' +
                   'AGAAAGAACGTAATGGTTTCTTGTCAGAAGTCCAGTGGTGGTCCTG' +
                   'AAGCAGCGGCTtgaaaa')
    reference_rna = RNA('AUGAUGGGCAGUGUCGAAUUAAAUCUGCGUGAGACAGAAUU' +
                        'GUGUUUGGGACUACCAGGCGGUGAUACAGUUGCACCAGUAA' +
                        'CAGGAAACAAAAGAGGAUUCUCUGAAACAGUAGAUUUGAAA' +
                        'CUUAAUUUGAACAAUGAGCCAGCCAACAAGGAAGGUUCCAC' +
                        'CACUCAUGACGUCGUCACAUUUGAUAGUAAAGAAAAGAGUG' +
                        'CGUGUCCAAAAGAUCCAGCUAAGCCACCUGCCAAGGCUCAA' +
                        'GUCGUCGGAUGGCCACCUGUGAGAUCUUAUAGAAAGAACGU' +
                        'AAUGGUUUCUUGUCAGAAGUCCAGUGGUGGUCCUGAAGCAG' +
                        'CGGCUugaaaa')
    # Basic transcription should work
    transcription_output = reaction.transcribe(test_dna)
    assert_equal(transcription_output, reference_rna)

    # Coding RNA should exclude anything after a stop codon
    coding_rna_output = reaction.coding_sequence(transcription_output)
    assert_equal(coding_rna_output, reference_rna[:-3])

    # Should fail is sequence lacks start codon or stop codon
    assert_raises(ValueError, reaction.coding_sequence,
                  reaction.transcribe(DNA('aaatag')))
    assert_raises(ValueError, reaction.coding_sequence,
                  reaction.transcribe(DNA('atgaaa')))


def test_translation():
    test_rna = RNA('AUGAUGGGCAGUGUCGAAUUAAAUCUGCGUGAGACAGAAUU' +
                   'GUGUUUGGGACUACCAGGCGGUGAUACAGUUGCACCAGUAA' +
                   'CAGGAAACAAAAGAGGAUUCUCUGAAACAGUAGAUUUGAAA' +
                   'CUUAAUUUGAACAAUGAGCCAGCCAACAAGGAAGGUUCCAC' +
                   'CACUCAUGACGUCGUCACAUUUGAUAGUAAAGAAAAGAGUG' +
                   'CGUGUCCAAAAGAUCCAGCUAAGCCACCUGCCAAGGCUCAA' +
                   'GUCGUCGGAUGGCCACCUGUGAGAUCUUAUAGAAAGAACGU' +
                   'AAUGGUUUCUUGUCAGAAGUCCAGUGGUGGUCCUGAAGCAG' +
                   'CGGCUugaaaa')
    reference_peptide = Peptide('MMGSVELNLRETELCLGLPGGDTVAPVTGNK' +
                                'RGFSETVDLKLNLNNEPANKEGSTTHDVVTF' +
                                'DSKEKSACPKDPAKPPAKAQVVGWPPVRSYR' +
                                'KNVMVSCQKSSGGPEAAA')
    # Basic transcription should work
    translation_output = reaction.translate(test_rna)
    assert_equal(translation_output, reference_peptide)

    # Coding peptide should exclude anything after a stop codon
    coding_rna = reaction.coding_sequence(test_rna)
    coding_peptide = reaction.translate(coding_rna)
    assert_equal(coding_peptide, reference_peptide)


def test_reverse_transcription():
    test_rna = RNA('AUGAUGGGCAGUGUCGAAUUAAAUCUGCGUGAGACAGAAUU' +
                   'GUGUUUGGGACUACCAGGCGGUGAUACAGUUGCACCAGUAA' +
                   'CAGGAAACAAAAGAGGAUUCUCUGAAACAGUAGAUUUGAAA' +
                   'CUUAAUUUGAACAAUGAGCCAGCCAACAAGGAAGGUUCCAC' +
                   'CACUCAUGACGUCGUCACAUUUGAUAGUAAAGAAAAGAGUG' +
                   'CGUGUCCAAAAGAUCCAGCUAAGCCACCUGCCAAGGCUCAA' +
                   'GUCGUCGGAUGGCCACCUGUGAGAUCUUAUAGAAAGAACGU' +
                   'AAUGGUUUCUUGUCAGAAGUCCAGUGGUGGUCCUGAAGCAG' +
                   'CGGCUugaaaa')
    ref_dna = DNA('ATGATGGGCAGTGTCGAATTAAATCTGCGTGAGACAGAATTGTGTT' +
                  'TGGGACTACCAGGCGGTGATACAGTTGCACCAGTAACAGGAAACAA' +
                  'AAGAGGATTCTCTGAAACAGTAGATTTGAAACTTAATTTGAACAAT' +
                  'GAGCCAGCCAACAAGGAAGGTTCCACCACTCATGACGTCGTCACAT' +
                  'TTGATAGTAAAGAAAAGAGTGCGTGTCCAAAAGATCCAGCTAAGCC' +
                  'ACCTGCCAAGGCTCAAGTCGTCGGATGGCCACCTGTGAGATCTTAT' +
                  'AGAAAGAACGTAATGGTTTCTTGTCAGAAGTCCAGTGGTGGTCCTG' +
                  'AAGCAGCGGCTtgaaaa')

    # Basic transcription should work
    r_transcription = reaction.reverse_transcribe(test_rna)
    assert_equal(r_transcription, ref_dna)



'''Test restriction reaction module.'''

from nose.tools import assert_equal
from coral import reaction, DNA, RestrictionSite


class TestDigest(object):
    '''Test digest function.'''
    def __init__(self):
        # Contains NcoI site
        self.dna = DNA('TGACCATGGAAA')

    def test_not_found(self):
        '''If site not found, should return input sequence in list.'''
        ecorv = RestrictionSite(DNA('GATATC'), (3, 3), name='EcoRV')
        assert_equal(self.dna, reaction.digest(self.dna, ecorv)[0])

    def test_ncoi_cut(self):
        '''Test standard TypeII cutter.'''
        ncoi = RestrictionSite(DNA('CCATGG'), (1, 5), name='NcoI')
        assert_equal(reaction.digest(self.dna, ncoi),
                     [DNA('TGAC----', bottom='CATGGTCA'),
                      DNA('CATGGAAA', bottom='TTTC----')])
        assert_equal(reaction.digest(self.dna.circularize(), ncoi),
                     [DNA('CATGGAAATGAC----', bottom='CATGGTCATTTC----')])

    def test_ecorv_cut(self):
        '''Test blunt-end cutter.'''
        ecorv = RestrictionSite(DNA('GATATC'), (3, 3), name='EcoRV')
        assert_equal(reaction.digest(DNA('GATATC'), ecorv),
                     [DNA('GAT'), DNA('ATC')])

    def test_psti_cut(self):
        '''Test 3\' cutter.'''
        psti = RestrictionSite(DNA('CTGCAG'), (5, 1), name='PstI')
        assert_equal(reaction.digest(DNA('ACTGCAGA'), psti),
                     [DNA('ACTGCA', bottom='----GT'),
                      DNA('----GA', bottom='TCTGCA')])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Builds sphinx docs for coral."""
import os
import subprocess


DOCSDIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "../docs"))


def build_docs(directory):
    """Builds sphinx docs from a given directory."""
    os.chdir(directory)
    process = subprocess.Popen(["make", "html"], cwd=directory)
    process.communicate()


if __name__ == "__main__":
    # Build sphinx docs (produces html)
    build_docs(DOCSDIR)



# Tests to write:
#
# analysis:
# TODO: write nupack_multiprocessing tests
# TODO: write Sanger tests
#
# reaction:
# TODO: write ReverseTranscription tests
# TODO: write Transcription tests
# TODO: write Translation tests
# TODO: write Restriction tests
#
# seqio:
# TODO: write read_dna tests
# TODO: write read_seq tests
#
# sequence:
# TODO: dna:
# TODO:   write tests for every public DNA method
# TODO:   write tests for RestrictionSite
# TODO: rna:
# TODO:   write tests for every public RNA method
# TODO: peptide:
# TODO:   write tests for every public Peptide method



'''RNA sequences classes.'''
import coral.reaction
from coral.sequence._nucleicacid import NucleicAcid


class RNA(NucleicAcid):
    '''ssRNA sequence.'''

    def __init__(self, rna, circular=False, run_checks=True):
        '''
        :param rna: Input sequence (RNA).
        :type rna: str
        :param run_checks: Check inputs / formats (disabling increases speed):
                           alphabet check
                           case
        :type run_checks: bool
        :returns: coral.RNA instance.

        '''
        super(RNA, self).__init__(rna, 'rna', circular=circular,
                                  run_checks=run_checks, any_char='N')
        self.ds = False

    def copy(self):
        return type(self)(self.seq, circular=self.circular, run_checks=False)

    def reverse_transcribe(self):
        '''Reverse transcribe to DNA.

        :returns: The reverse transcribed (DNA) version of the current RNA.
        :rtype: coral.DNA

        '''
        return coral.reaction.reverse_transcribe(self)

    def translate(self):
        '''Translate sequence into a peptide.

        :returns: A translated peptide from the current sequence.
        :rtype: coral.Peptide

        '''
        return coral.reaction.translate(self)



'''
Tests for the Peptide sequence class.

'''

from coral import Peptide
from nose.tools import assert_equal, assert_false, assert_true, assert_raises


class TestPeptide(object):
    '''
    Testing class for sequence.Peptide

    '''

    def __init__(self):
        self.test_peptide = Peptide('mkgp')

    def test_locate(self):
        assert_equal(self.test_peptide.locate('mk'), [0])
        assert_equal(self.test_peptide.locate('gp'), [2])
        assert_equal(len(self.test_peptide.locate('augg')), 0)

    def test_copy(self):
        assert_equal(self.test_peptide, self.test_peptide.copy())

    def test_getitem(self):
        assert_equal(str(self.test_peptide[0]), 'M')
        assert_equal(str(self.test_peptide[1]), 'K')
        assert_equal(str(self.test_peptide[2]), 'G')
        assert_equal(str(self.test_peptide[3]), 'P')
        assert_equal(str(self.test_peptide[-1]), 'P')

    def test_delitem(self):
        copy0 = self.test_peptide.copy()
        del copy0[0]
        assert_equal(str(copy0), 'KGP')
        copy1 = self.test_peptide.copy()
        del copy1[1]
        assert_equal(str(copy1), 'MGP')
        copy2 = self.test_peptide.copy()
        del copy2[2]
        assert_equal(str(copy2), 'MKP')
        copy3 = self.test_peptide.copy()
        del copy3[3]
        assert_equal(str(copy3), 'MKG')
        copy_1 = self.test_peptide.copy()
        del copy_1[-1]
        assert_equal(str(copy_1), 'MKG')

    def test_setitem(self):
        copy0 = self.test_peptide.copy()
        copy0[0] = 'q'
        assert_equal(str(copy0), 'QKGP')
        copy1 = self.test_peptide.copy()
        copy1[1] = 'q'
        assert_equal(str(copy1), 'MQGP')
        copy2 = self.test_peptide.copy()
        copy2[2] = 'q'
        assert_equal(str(copy2), 'MKQP')
        copy3 = self.test_peptide.copy()
        copy3[3] = 'q'
        assert_equal(str(copy3), 'MKGQ')
        copy_1 = self.test_peptide.copy()
        copy_1[-1] = 'q'
        assert_equal(str(copy_1), 'MKGQ')

    def test_str(self):
        assert_equal(str(self.test_peptide), 'MKGP')

    def test_len(self):
        assert_equal(len(self.test_peptide), 4)

    def test_add(self):
        assert_equal(str((self.test_peptide + self.test_peptide)),
                     'MKGPMKGP')

    def test_radd(self):
        assert_equal(str(sum([self.test_peptide, self.test_peptide])),
                     'MKGPMKGP')

        def radd_800(seq):
            return 800 + seq

        assert_raises(TypeError, radd_800, self.test_peptide)

    def test_mul(self):
        assert_equal(str(self.test_peptide * 4), 'MKGPMKGPMKGPMKGP')

        def mul_float(seq):
            return seq * 7.56

        assert_raises(TypeError, mul_float, self.test_peptide)

    def test_eq(self):
        assert_true(self.test_peptide == Peptide('mkgp'))

    def test_ne(self):
        assert_true(self.test_peptide != Peptide('mkqp'))

    def test_contains(self):
        assert_true('m' in self.test_peptide)
        assert_true('k' in self.test_peptide)
        assert_true('g' in self.test_peptide)
        assert_true('p' in self.test_peptide)
        assert_false('q' in self.test_peptide)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Uses nbconvert to convert all ipynbs in the ipython_examples dir until I
can figure out a way to use a proper sphinx extension. This is really hacky and
should not be used as a secure production server."""
# TODO: catch conversion errors (right now they pass silently)
# Doesn't even use IPython API (TODO!)
import os
from tornado import web, ioloop, httpserver
from ipynb2rst import convert_ipynbs
from build_sphinx_docs import build_docs


DOCSDIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "../docs"))
ROOT = os.path.abspath(os.path.join(DOCSDIR, "_build/html"))
STATIC = os.path.join(ROOT, "_static")


# Web server (Tornado) classes
class MainHandler(web.RequestHandler):
    def get(self, path, *args):
        uri = self.request.uri
        if uri == "/" or uri == "index.html":
            self.render("index.html")
        else:
            self.render(self.request.uri[1:])


class Application(web.Application):
    def __init__(self):
        handlers = [(r"^(.(?!_static))*$", MainHandler),
                    (r"/_static/(.*)", web.StaticFileHandler,
                     {"path": STATIC})]
        settings = {"template_path": ROOT,
                    "static_url_prefix": "_static"}
        web.Application.__init__(self, handlers, **settings)


if __name__ == "__main__":
    # Convert notebooks from ipynb to rst
    convert_ipynbs(DOCSDIR)
    # Build sphinx docs (produces html)
    build_docs(DOCSDIR)
    # Launch server
    applicaton = Application()
    http_server = httpserver.HTTPServer(applicaton)
    http_server.listen(3089)

    ioloop.IOLoop.instance().start()



'''Test functionality of PCR class of reaction module.'''
from nose.tools import assert_equal, assert_raises
import coral as cr


# TODO: refactor into class that shares sequence reading, etc.
class TestPCR(object):
    '''Testing class for reaction.pcr.'''
    def __init__(self):
        # Part BBa_R0010 (pLac promoter)
        bba_r0010 = ('caatacgcaaaccgcctctccccgcgcgttggccgattcattaatgcag'
                     'ctggcacgacaggtttcccgactggaaagcgggcagtgagcgcaacgca'
                     'attaatgtgagttagctcactcattaggcaccccaggctttacacttta'
                     'tgcttccggctcgtatgttgtgtggaattgtgagcggataacaatttca'
                     'caca')
        self.template = cr.DNA(bba_r0010, circular=False)

    def pcr_equal(self, expected, template, primer1, primer2):
        '''Boilerplate to assert that a pcr reaction matches the expected
        sequence.'''
        amplicon = cr.reaction.pcr(template, primer1, primer2)
        assert_equal(expected, amplicon)

    '''
    Test all the different scenarios:
    Linear template:
      1. Primers point towards each other (whole amplicon and subsets)
      2. Extract region contained between them + their sizes
    Circular template (primers can point towards or away from each other):
      3. Origin (index 0) is between the primers
      4. Origin (index 0) isn't involved at all (amplicon outside it)
      5. Origin isn't between the primers, but forward primer overlaps it.
      6. Origin isn't between the primers, but reverse primer overlaps it.
      7. Origin isn't between the primers, but both primers overlap it.
    '''
    def test_linear_towards_one_another(self):
        '''Amplify entire template, no overhangs.'''
        fwd, rev = cr.design.primers(self.template)
        self.pcr_equal(self.template, self.template, fwd, rev)

    def test_subset(self):
        '''Amplify part of the template, no overhangs.'''
        fwd, rev = cr.design.primers(self.template[30:-30])
        self.pcr_equal(self.template[30:-30], self.template, fwd, rev)

    def test_linear_away_from_one_another(self):
        '''If primers point away from each other on a linear template, raise
        eception.'''
        fwd = cr.design.primer(self.template[-60:])
        rev = cr.design.primer(self.template[:60].reverse_complement())
        assert_raises(cr.reaction._pcr.PrimingError, cr.reaction.pcr,
                      self.template, fwd, rev)

    def test_circular_over_origin(self):
        '''Amplify a circular template over the origin, no overhangs.'''
        template = self.template.circularize()
        fwd = cr.design.primer(template[-60:])
        rev = cr.design.primer(template[:60].reverse_complement())
        expected = template[-60:] + template[:60]
        self.pcr_equal(expected, template, fwd, rev)

    def test_circular_outside_origin(self):
        '''Amplify entire template, no overhangs.'''
        template = self.template.circularize()
        fwd, rev = cr.design.primers(template[30:-30])
        self.pcr_equal(self.template[30:-30], template, fwd, rev)

    def test_circular_fwd_overlap(self):
        '''Test for when the forward primer overlaps the origin (index 0).'''
        template = self.template.circularize()
        fwd = cr.design.primer(template.rotate(4))
        rev = cr.design.primer(template[:60].reverse_complement())
        expected = template[-4:] + template[:60]
        self.pcr_equal(expected, template, fwd, rev)

    def test_circular_rev_overlap(self):
        '''Test for when the reverse primer overlaps the origin (index 0).'''
        template = self.template.circularize()
        fwd = cr.design.primer(template[60:])
        rev = cr.design.primer(template.rotate(-4).reverse_complement())
        expected = template[60:] + template[:4]
        self.pcr_equal(expected, template, fwd, rev)

    def test_circular_fwd_rev_overlap(self):
        '''Test for when both primers overlap the origin (index 0).'''
        template = self.template.circularize()
        fwd = cr.design.primer(template.rotate(4))
        rev = cr.design.primer(template.rotate(-4).reverse_complement())
        expected = template[-4:] + template.linearize() + template[:4]
        self.pcr_equal(expected, template, fwd, rev)

    def test_primer_order(self):
        '''Amplification should occur regardless of the order in which primers
        are specified.'''
        fwd, rev = cr.design.primers(self.template)
        fwd, rev = rev, fwd
        self.pcr_equal(self.template, self.template, fwd, rev)

    def test_primers_overlap(self):
        '''Tests case where primers overlap.'''
        # ~10 bp of overlap
        fwd_seq = self.template[100:120]
        rev_seq = self.template[110:130].reverse_complement()
        fwd = cr.Primer(fwd_seq, fwd_seq.tm())
        rev = cr.Primer(rev_seq, rev_seq.tm())
        # Should amplify 30 bp region
        self.pcr_equal(self.template[100:130], self.template, fwd, rev)

    def test_overhang(self):
        '''Tests that primer overhangs are added correctly to the amplicon.'''
        template = self.template[30:-30]
        fwd_overhang = cr.DNA('AGCGGGGGGGGGCTGGGGCTGAT')
        rev_overhang = cr.DNA('GGGTGGGGGGGGGGGGGGG')
        fwd = cr.design.primer(template, overhang=fwd_overhang)
        rev = cr.design.primer(template.reverse_complement(),
                               overhang=rev_overhang)
        expected = (fwd_overhang +
                    template +
                    rev_overhang.reverse_complement())
        self.pcr_equal(expected, self.template, fwd, rev)

    def test_bad_primer2(self):
        '''Test that a bad primer raises a PrimingError.'''
        fwd, rev = cr.design.primers(self.template)
        rev.anneal[-10:] = 'AAAAAAAAAA'
        assert_raises(cr.reaction._pcr.PrimingError, cr.reaction.pcr,
                      self.template, fwd, rev)

    def test_primers_in_same_direction(self):
        '''Test that primers that bind the same strand raise an error.'''
        fwd1 = cr.design.primer(self.template)
        fwd2 = cr.design.primer(self.template[50:])
        assert_raises(cr.reaction._pcr.PrimingError, cr.reaction.pcr,
                      self.template, fwd1, fwd2)

    def test_ambiguous_priming(self):
        '''Test that ambiguous primer binding sites raises an error.'''
        template = self.template[:50] + self.template
        fwd, rev = cr.design.primers(template)
        assert_raises(cr.reaction._pcr.PrimingError, cr.reaction.pcr,
                      template, fwd, rev)



'''
Tests for RandomBases class of analysis module.

'''

from nose.tools import assert_equal
from coral import design


def test_randomdna():
    '''
    This test is pretty basic right now - not sure how much checking
    can be done for a random DNA base generator.

    '''

    output = design.random_dna(200)
    assert_equal(len(output), 200)



'''Base sequence classes.'''
import collections
import coral
from coral.sequence._sequence import Sequence
from coral.constants.molecular_bio import COMPLEMENTS


class NucleicAcid(Sequence):
    '''Abstract sequence container for a single nucleic acid sequence
    molecule.'''

    def __init__(self, sequence, material, circular=False, run_checks=True,
                 any_char='N'):
        '''
        :param sequence: Input sequence.
        :type sequence: str
        :param material: Material type (dna, rna)
        :type material: str
        :param circular: The topology of the sequence - if the ends connect,
                         (a circular sequence), set to True. Otherwise, set to
                         False. Enables operations like .rotate().
        :type circular: bool
        :param run_checks: Check inputs / formats (disabling increases speed):
                           alphabet check
                           case
        :param any_char: Character representing \'any\', e.g. N for DNA.
        :type any_char: str
        :type run_checks: bool
        :returns: coral.sequence.Sequence instance.

        '''
        super(NucleicAcid, self).__init__(sequence, material,
                                          run_checks=run_checks,
                                          any_char=any_char)
        self.circular = circular

    def copy(self):
        return type(self)(self.seq, self.material, circular=self.circular,
                          run_checks=False)

    def circularize(self):
        '''Circularize the sequence, if linear.

        :returns: A circularized version of the current sequence.
        :rtype: coral.sequence._sequence.Sequence

        '''
        copy = self.copy()
        copy.circular = True
        return copy

    def complement(self):
        copy = self.copy()
        code = dict(COMPLEMENTS[self.material])
        copy.seq = ''.join([code[str(base)] for base in copy])
        return copy

    def gc(self):
        '''Find the frequency of G and C in the current sequence.'''
        gc = len([base for base in self.seq if base == 'C' or base == 'G'])
        return float(gc) / len(self)

    def is_palindrome(self):
        seq_len = len(self.seq)
        if seq_len % 2 == 0:
            # Sequence has even number of bases, can test non-overlapping seqs
            wing = seq_len / 2
            l_wing = self[0: wing]
            r_wing = self[wing:]
            if l_wing == r_wing.reverse_complement():
                return True
            else:
                return False
        else:
            # Sequence has odd number of bases and cannot be a palindrome
            return False

    def is_rotation(self, other):
        '''Determine whether two sequences are the same, just at different
        rotations.

        :param other: The sequence to check for rotational equality.
        :type other: coral.sequence._sequence.Sequence

        '''
        if len(self) != len(other):
            return False

        for i in range(len(self)):
            if self.rotate(i) == other:
                return True

        return False

    def linearize(self, index=0):
        '''Linearize the Sequence at an index.

        :param index: index at which to linearize.
        :type index: int
        :returns: A linearized version of the current sequence.
        :rtype: coral.sequence._sequence.Sequence
        :raises: ValueError if the input is a linear sequence.

        '''
        if not self.circular and index != 0:
            raise ValueError('Cannot relinearize a linear sequence.')
        copy = self.copy()
        # Snip at the index
        if index:
            return copy[index:] + copy[:index]
        copy.circular = False

        return copy

    def locate(self, pattern):
        '''Find sequences matching a pattern.

        :param pattern: Sequence for which to find matches.
        :type pattern: str
        :returns: Indices of pattern matches.
        :rtype: list of ints

        '''
        if self.circular:
            if len(pattern) >= 2 * len(self):
                raise ValueError('Search pattern longer than searchable ' +
                                 'sequence.')
            seq = self + self[:len(pattern) - 1]
            return super(NucleicAcid, seq).locate(pattern)
        else:
            return super(NucleicAcid, self).locate(pattern)

    def mw(self):
        '''Calculate the molecular weight.

        :returns: The molecular weight of the current sequence in amu.
        :rtype: float

        '''
        counter = collections.Counter(self.seq.lower())
        mw_a = counter['a'] * 313.2
        mw_t = counter['t'] * 304.2
        mw_g = counter['g'] * 289.2
        mw_c = counter['c'] * 329.2
        mw_u = counter['u'] * 306.2

        if self.material == 'dna':
            return mw_a + mw_t + mw_g + mw_c + 79.0
        else:
            return mw_a + mw_u + mw_g + mw_c + 159.0

    def rotate(self, n):
        '''Rotate Sequence by n bases.

        :param n: Number of bases to rotate.
        :type n: int
        :returns: The current sequence reoriented at `index`.
        :rtype: coral.sequence._sequence.Sequence
        :raises: ValueError if applied to linear sequence or `index` is
                 negative.

        '''
        if not self.circular and n != 0:
            raise ValueError('Cannot rotate a linear sequence')
        else:
            rotated = self[-n:] + self[:-n]
            return rotated.circularize()

    def rotate_to(self, index):
        '''Orient Sequence to index (only applies to circular sequences).

        :param index: Position at which to re-zero the Sequence.
        :type index: int
        :returns: The current sequence reoriented at `index`.
        :rtype: coral.sequence._sequence.Sequence
        :raises: ValueError if applied to linear sequence or `index` is
                 negative.

        '''
        return self.rotate(-index)

    def reverse(self):
        return self[::-1]

    def reverse_complement(self):
        copy = self.copy()
        copy.seq = str(self.reverse().complement())
        return copy

    def tm(self, parameters='cloning'):
        '''Find the melting temperature.

        :param parameters: The tm method to use (cloning, santalucia98,
                       breslauer86)
        :type parameters: str

        '''
        return coral.analysis.tm(self, parameters=parameters)



'''Sanger sequencing alignment tools.'''
import coral as cr

# FIXME: sequencing that goes past 'end' of a circular reference
# is reported as an insertion
# TODO: consensus / master sequence for plotting / report / analysis


class Sanger(object):
    '''Align and analyze Sanger sequencing results.'''

    def __init__(self, reference, results, method='needle',
                 method_kwargs=None):
        '''
        :param reference: Reference sequence.
        :type reference: :class:`coral.DNA`
        :param results: Sequencing result string. A list of DNA objects is also
                        valid.
        :type results: list of coral.DNA sequences
        :param method: Alignment method to use. Options are:
                         \'needle\': Uses coral.analysis.needle_msa
                         \'MAFFT\': Uses coral.analysis.MAFFT
        :type method: str
        :param method_kwargs: Optional keyword arguments to send to the
                              alignment function.
        :type method_kwargs: dict
        :returns: instance of coral.analysis.Sanger (contains alignment and
                  provides analysis/visualization methods

        '''
        # Alignment params / thresholds
        # self._gap_open = -25
        # self._gap_extend = 0
        # self._score_threshold = 100

        # Sequences and calculations that get reused
        self.reference = reference
        self.results = results
        self.method = method
        if method_kwargs is None:
            self.method_kwargs = {}
        else:
            self.method_kwargs = method_kwargs

        self._remove_n()
        self.names = []
        for i, result in enumerate(self.results):
            if not result.name:
                self.names.append('Result {}'.format(i))
            else:
                self.names.append(result.name)

        # Align
        self.align()

    def align(self):
        if self.method == 'needle':
            self.alignment = cr.analysis.needle_msa(self.reference,
                                                    self.results,
                                                    **self.method_kwargs)
        elif self.method == 'MAFFT':
            self.alignment = cr.analysis.MAFFT([self.reference] + self.results,
                                               **self.method_kwargs)
        else:
            raise ValueError('Only \'needle\' or \'MAFFT\' methods allowed.')

        self.aligned_reference = self.alignment[0].copy()
        self.aligned_results = []
        for result in self.alignment[1:]:
            self.aligned_results.append(result.copy())

    def nonmatches(self):
        '''Report mismatches, indels, and coverage.'''
        # For every result, keep a dictionary of mismatches, insertions, and
        # deletions
        report = []
        for result in self.aligned_results:
            report.append(self._analyze_single(self.aligned_reference, result))

        return report

    def plot(self):
        '''Make a summary plot of the alignment and highlight nonmatches.'''
        import matplotlib.pyplot as plt
        import matplotlib.patches as patches

        # Constants to use throughout drawing
        n = len(self.results)
        nbases = len(self.aligned_reference)
        barheight = 0.4

        # Vary height of figure based on number of results
        figheight = 3 + 3 * (n - 1)
        fig = plt.figure(figsize=(9, figheight))
        ax1 = fig.add_subplot(111)

        # Plot bars to represent coverage area
        # Reference sequence
        ax1.add_patch(patches.Rectangle((0, 0), nbases, barheight,
                                        facecolor='black'))
        # Results
        for i, report in enumerate(self.nonmatches()):
            j = i + 1
            start, stop = report['coverage']
            patch = patches.Rectangle((start, j), stop - start, barheight,
                                      facecolor='darkgray')
            ax1.add_patch(patch)

            # Draw a vertical line for each type of result
            plt.vlines(report['mismatches'], j, j + barheight,
                       colors='b')
            plt.vlines(report['insertions'], j, j + barheight,
                       colors='r')

            # Terminal trailing deletions shouldn't be added
            deletions = []
            crange = range(*report['coverage'])
            deletions = [idx for idx in report['deletions'] if idx in crange]
            plt.vlines(deletions, j, j + barheight,
                       colors='g')

        ax1.set_xlim((0, nbases))
        ax1.set_ylim((-0.3, n + 1))
        ax1.set_yticks([i + barheight / 2 for i in range(n + 1)])
        ax1.set_yticklabels(['Reference'] + self.names)

        # Add legend
        mismatch_patch = patches.Patch(color='blue', label='Mismatch')
        insertion_patch = patches.Patch(color='red', label='Insertion')
        deletion_patch = patches.Patch(color='green', label='Deletion')
        plt.legend(handles=[mismatch_patch, insertion_patch, deletion_patch],
                   loc=1, ncol=3, mode='expand', borderaxespad=0.)

        plt.show()

    def _analyze_single(self, reference, result):
        '''Report mistmatches and indels for a single (aligned) reference and
        result.'''
        # TODO: Recalculate coverage based on reference (e.g. sequencing result
        # longer than template
        reference_str = str(reference)
        result_str = str(result)
        report = {'mismatches': [], 'insertions': [], 'deletions': []}
        for i, (ref, res) in enumerate(zip(reference_str, result_str)):
            if ref != res:
                # It's a mismatch or indel
                if ref == '-':
                    report['insertions'].append(i)
                if res == '-':
                    report['deletions'].append(i)
                else:
                    report['mismatches'].append(i)

        start = len(result_str) - len(result_str.lstrip('-'))
        stop = len(result_str) - len(result_str.rstrip('-'))
        report['coverage'] = [start, stop]

        return report

    def _remove_n(self):
        '''Remove terminal Ns from sequencing results.'''
        for i, result in enumerate(self.results):
            largest = max(str(result).split('N'), key=len)
            start = result.locate(largest)[0][0]
            stop = start + len(largest)
            if start != stop:
                self.results[i] = self.results[i][start:stop]



'''
Tests for the OligoAssembly design class.

'''

from nose.tools import assert_equal
from coral import design, DNA


def test_oligo_assembly():
    '''
    Tests output of OligoAssembly class.

    '''

    # Expected outputs
    olig1 = 'ATGCGTAAAGGAGAAGAACTTTTCACTGGAGTTGTCCCAATTCTTGTTGAATTAGATGGTG' + \
            'ATGTTAATGGGCACAAATTTTCTGTCAGTGGAGAGGGTGAA'
    olig2 = 'TGGCCATGGAACAGGTAGTTTTCCAGTAGTGCAAATAAATTTAAGGGTAAGTTTTCCGTAT' + \
            'GTTGCATCACCTTCACCCTCTCCACTGACAGAAAATTTGTG'
    olig3 = 'TGGAAAACTACCTGTTCCATGGCCAACACTTGTCACTACTTTCGGTTATGGTGTTCAATGC' + \
            'TTTGCGAGATACCCAGATCATATGAAACAGCATGACTTTTTCAA'
    olig4 = 'CGTGTCTTGTAGTTCCCGTCATCTTTGAAAAATATAGTTCTTTCCTGTACATAACCTTCGG' + \
            'GCATGGCACTCTTGAAAAAGTCATGCTGTTTCATATGATCTGGG'
    olig5 = 'TTCAAAGATGACGGGAACTACAAGACACGTGCTGAAGTCAAGTTTGAAGGTGATACCCTTG' + \
            'TTAATAGAATCGAGTTAAAAGGTATTGATTTTAAAGAAGATGGAAACA'
    olig6 = 'TTTGTCTGCCATGATGTATACATTGTGTGAGTTATAGTTGTATTCCAATTTGTGTCCAAGA' + \
            'ATGTTTCCATCTTCTTTAAAATCAATACCTTTTAACTCGATTCTATT'
    olig7 = 'AACTATAACTCACACAATGTATACATCATGGCAGACAAACAAAAGAATGGAATCAAAGTTA' + \
            'ACTTCAAAATTAGACACAACATTGAAGATGGAAGCGTTCAACTAGCA'
    olig8 = 'TTGTGTGGACAGGTAATGGTTGTCTGGTAAAAGGACAGGGCCATCGCCAATTGGAGTATTT' + \
            'TGTTGATAATGGTCTGCTAGTTGAACGCTTCCATCTTCAATGT'
    olig9 = 'CCAGACAACCATTACCTGTCCACACAATCTGCCCTTTCGAAAGATCCCAACGAAAAGAGAG' + \
            'ACCACATGGTCCTTCTTGAGTTTGTAACAGCTGCTGGGA'
    olig10 = 'TTAAGCTACTAAAGCGTAGTTTTCGTCGTTTGCAGCAGGCCTTTTGTATAGTTCATCCAT' + \
             'GCCATGTGTAATCCCAGCAGCTGTTACAAACTCAAGAAGG'

    reference_oligos = [olig1, olig2, olig3, olig4, olig5, olig6, olig7, olig8,
                        olig9, olig10]
    reference_tms = [73.513413945987, 72.73367624289534, 73.73563193690484,
                     72.70706564878299, 72.72193323127533, 72.23050918438184,
                     72.07546311550101, 72.27046461560099, 73.67230272019759]

    # Run oligo synthesis on BBa_K082003
    seq = 'atgcgtaaaggagaagaacttttcactggagttgtcccaattcttgttgaattagatggtgat' + \
          'gttaatgggcacaaattttctgtcagtggagagggtgaaggtgatgcaacatacggaaaactt' + \
          'acccttaaatttatttgcactactggaaaactacctgttccatggccaacacttgtcactact' + \
          'ttcggttatggtgttcaatgctttgcgagatacccagatcatatgaaacagcatgactttttc' + \
          'aagagtgccatgcccgaaggttatgtacaggaaagaactatatttttcaaagatgacgggaac' + \
          'tacaagacacgtgctgaagtcaagtttgaaggtgatacccttgttaatagaatcgagttaaaa' + \
          'ggtattgattttaaagaagatggaaacattcttggacacaaattggaatacaactataactca' + \
          'cacaatgtatacatcatggcagacaaacaaaagaatggaatcaaagttaacttcaaaattaga' + \
          'cacaacattgaagatggaagcgttcaactagcagaccattatcaacaaaatactccaattggc' + \
          'gatggccctgtccttttaccagacaaccattacctgtccacacaatctgccctttcgaaagat' + \
          'cccaacgaaaagagagaccacatggtccttcttgagtttgtaacagctgctgggattacacat' + \
          'ggcatggatgaactatacaaaaggcctgctgcaaacgacgaaaactacgctttagtagcttaa'
    dna_seq = DNA(seq)
    assembly = design.OligoAssembly(dna_seq,
                                    tm=72,
                                    length_range=(120, 120),
                                    require_even=True,
                                    start_5=True)
    assembly.design_assembly()

    # Prepare outputs vs reference
    output_oligos = [str(oligo).lower() for oligo in assembly.oligos]
    reference_oligos = [oligo.lower() for oligo in reference_oligos]

    assert_equal(output_oligos, reference_oligos)
    assert_equal(assembly.overlap_tms, reference_tms)

    # Test too short of oligo input
    too_short = DNA(seq[0:100])
    too_short_assembly = design.OligoAssembly(too_short,
                                              tm=72,
                                              length_range=(120, 120),
                                              require_even=True,
                                              start_5=True)
    too_short_assembly.design_assembly()
    assert_equal(str(too_short_assembly.oligos[0]), str(too_short))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Uses nbconvert to recursively convert all ipynbs in a directory to .rst."""
# TODO: catch conversion errors (right now they pass silently)
# Doesn't even use IPython API (TODO!)
import os
import subprocess
import sys


# Build docs
def ipynb_to_rst(directory, filename):
    """Converts a given file in a directory to an rst in the same directory."""
    os.chdir(directory)
    subprocess.Popen(["ipython", "nbconvert", "--to", "rst",
                      filename],
                     stdout=subprocess.PIPE,
                     stderr=subprocess.PIPE,
                     cwd=directory)


def convert_ipynbs(directory):
    """Recursively converts all ipynb files in a directory into rst files in
    the same directory."""
    # The ipython_examples dir has to be in the same dir as this script
    for root, subfolders, files in os.walk(os.path.abspath(directory)):
        for f in files:
            if ".ipynb_checkpoints" not in root:
                if f.endswith("ipynb"):
                    ipynb_to_rst(root, f)


if __name__ == "__main__":
    # Convert notebooks from ipynb to rst
    if len(sys.argv) != 2:
        print "\nipy2nb:\n=======\nUsage: ipynb2rst <directory>\n"
    else:
        directory = sys.argv[1]
        convert_ipynbs(directory)



'''
Tests Repeats analysis class.

'''

from nose.tools import assert_equal
from coral import analysis, DNA


def test_find_repeats():
    input_sequence = DNA('atgatgccccgatagtagtagtag')
    expected = [('ATG', 2), ('GTA', 3), ('GAT', 2), ('AGT', 3), ('CCC', 2),
                ('TAG', 4)]

    output = analysis.repeats(input_sequence, 3)
    assert_equal(output, expected)



'''A Coral wrapper for the MAFFT command line multiple sequence aligner.'''
import coral
import os
import shutil
import subprocess
import tempfile


def MAFFT(sequences, gap_open=1.53, gap_extension=0.0, retree=2):
    '''A Coral wrapper for the MAFFT command line multiple sequence aligner.

    :param sequences: A list of sequences to align.
    :type sequences: List of homogeneous sequences (all DNA, or all RNA,
                     etc.)
    :param gap_open: --op (gap open) penalty in MAFFT cli.
    :type gap_open: float
    :param gap_extension: --ep (gap extension) penalty in MAFFT cli.
    :type gap_extension: float
    :param retree: Number of times to build the guide tree.
    :type retree: int

    '''
    arguments = ['mafft']
    arguments += ['--op', str(gap_open)]
    arguments += ['--ep', str(gap_extension)]
    arguments += ['--retree', str(retree)]
    arguments.append('input.fasta')
    tempdir = tempfile.mkdtemp()
    try:
        with open(os.path.join(tempdir, 'input.fasta'), 'w') as f:
            for i, sequence in enumerate(sequences):
                if hasattr(sequence, 'name'):
                    name = sequence.name
                else:
                    name = 'sequence{}'.format(i)
                f.write('>{}\n'.format(name))
                f.write(str(sequence) + '\n')
        process = subprocess.Popen(arguments, stdout=subprocess.PIPE,
                                   stderr=open(os.devnull, 'w'), cwd=tempdir)
        stdout = process.communicate()[0]
    finally:
        shutil.rmtree(tempdir)

    # Process stdout into something downstream process can use

    records = stdout.split('>')
    # First line is now blank
    records.pop(0)
    aligned_list = []
    for record in records:
        lines = record.split('\n')
        name = lines.pop(0)
        aligned_list.append(coral.DNA(''.join(lines)))

    return aligned_list



'''
Tests for the Tm analysis class.

'''

from nose.tools import assert_equal
from coral import analysis, DNA


def test_finnzymes():
    '''
    Tests finnzymes method output.

    '''

    melt = analysis.tm(DNA('ATGCGATAGCGATAGC'), parameters='cloning')
    assert_equal(melt, 55.2370030020752)
